@@ -27,10 +27,10 @@ event 2 has 1 records
         syscall=188 (setxattr)
         success=yes (yes)
         exit=0 (0)
-        a0=7fffffa9a9f0 (7fffffa9a9f0)
-        a1=3958d11333 (3958d11333)
-        a2=5131f0 (5131f0)
-        a3=20 (20)
+        a0=7fffffa9a9f0 (0x7fffffa9a9f0)
+        a1=3958d11333 (0x3958d11333)
+        a2=5131f0 (0x5131f0)
+        a3=20 (0x20)
         items=1 (1)
         pid=2027 (2027)
         auid=848 (unknown(848))
@@ -97,10 +97,10 @@ event 1 has 4 records
         syscall=2 (open)
         success=no (no)
         exit=-13 (-13(Permission denied))
-        a0=5555665d91b0 (5555665d91b0)
+        a0=5555665d91b0 (0x5555665d91b0)
         a1=10800 (O_RDONLY|O_NONBLOCK|O_DIRECTORY)
-        a2=5555665d91b8 (5555665d91b8)
-        a3=0 (0)
+        a2=5555665d91b8 (0x5555665d91b8)
+        a3=0 (0x0)
         items=1 (1)
         ppid=2013 (2013)
         pid=13010 (13010)
@@ -256,10 +256,10 @@ event 1 has 4 records
         syscall=2 (open)
         success=no (no)
         exit=-13 (-13(Permission denied))
-        a0=5555665d91b0 (5555665d91b0)
+        a0=5555665d91b0 (0x5555665d91b0)
         a1=10800 (O_RDONLY|O_NONBLOCK|O_DIRECTORY)
-        a2=5555665d91b8 (5555665d91b8)
-        a3=0 (0)
+        a2=5555665d91b8 (0x5555665d91b8)
+        a3=0 (0x0)
         items=1 (1)
         ppid=2013 (2013)
         pid=13010 (13010)
@@ -412,10 +412,10 @@ event 8 has 4 records
         syscall=2 (open)
         success=no (no)
         exit=-13 (-13(Permission denied))
-        a0=5555665d91b0 (5555665d91b0)
+        a0=5555665d91b0 (0x5555665d91b0)
         a1=10800 (O_RDONLY|O_NONBLOCK|O_DIRECTORY)
-        a2=5555665d91b8 (5555665d91b8)
-        a3=0 (0)
+        a2=5555665d91b8 (0x5555665d91b8)
+        a3=0 (0x0)
         items=1 (1)
         ppid=2013 (2013)
         pid=13010 (13010)
@@ -593,10 +593,10 @@ event 2 has 1 records
         syscall=188 (setxattr)
         success=yes (yes)
         exit=0 (0)
-        a0=7fffffa9a9f0 (7fffffa9a9f0)
-        a1=3958d11333 (3958d11333)
-        a2=5131f0 (5131f0)
-        a3=20 (20)
+        a0=7fffffa9a9f0 (0x7fffffa9a9f0)
+        a1=3958d11333 (0x3958d11333)
+        a2=5131f0 (0x5131f0)
+        a3=20 (0x20)
         items=1 (1)
         pid=2027 (2027)
         auid=848 (unknown(848))
@@ -655,10 +655,10 @@ event 1 has 4 records
         syscall=2 (open)
         success=no (no)
         exit=-13 (-13(Permission denied))
-        a0=5555665d91b0 (5555665d91b0)
+        a0=5555665d91b0 (0x5555665d91b0)
         a1=10800 (O_RDONLY|O_NONBLOCK|O_DIRECTORY)
-        a2=5555665d91b8 (5555665d91b8)
-        a3=0 (0)
+        a2=5555665d91b8 (0x5555665d91b8)
+        a3=0 (0x0)
         items=1 (1)
         ppid=2013 (2013)
         pid=13010 (13010)
